#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-06-07
#
# monitor Juniper Networks craft alarms
#
# 2023-09-01: fixed empty string_table (THX to joerg[dot]ott[at]outlook[dot]com)
#             added WATO rule
#             added alarm state
#             added last change (notice only, needs sample with actual time)
#
# ToDo: show correct time for last change
#
# sample snmpwalk
# .1.3.6.1.4.1.2636.3.4.2.1.0 = INTEGER: 2
# .1.3.6.1.4.1.2636.3.4.2.2.1.0 = INTEGER: 2
# .1.3.6.1.4.1.2636.3.4.2.2.2.0 = Gauge32: 0
# .1.3.6.1.4.1.2636.3.4.2.2.3.0 = 0
# .1.3.6.1.4.1.2636.3.4.2.3.1.0 = INTEGER: 2
# .1.3.6.1.4.1.2636.3.4.2.3.2.0 = Gauge32: 0
# .1.3.6.1.4.1.2636.3.4.2.3.3.0 = 0
#
# sample string_table
# [[['2']], [['2', '0', '0']], [['2', '0', '0']]]
#
# sample section
# JuniperAlarms(
#   alarm_relay_mode=2,
#   yellow_alarm=JuniperAlarm(state=2, count=0, last_change=0),
#   red_alarm=JuniperAlarm(state=2, count=0, last_change=0)
#  )
#

from dataclasses import dataclass
from typing import Optional, List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    startswith,
    check_levels,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class JuniperAlarm:
    state: int
    count: int
    last_change: int


@dataclass
class JuniperAlarms:
    alarm_relay_mode: int
    yellow_alarm: JuniperAlarm
    red_alarm: JuniperAlarm


_juniper_alarm_relay_mode = {
    1: 'other',
    2: 'pass on',
    3: 'cut off',
}

_juniper_alarm_state = {
    1: 'other',
    2: 'off',
    3: 'on',
}


def parse_juniper_craft_alarms(string_table: List[StringTable]) -> Optional[JuniperAlarms]:
    try:
        alarm_relay_mode, yellow_alarm, red_alarm = string_table
    except IndexError:
        return

    try:
        alarm_relay_mode = alarm_relay_mode[0][0]
    except IndexError:
        return

    try:
        yellow_state, yellow_count, yellow_last_change = yellow_alarm[0]
    except IndexError:
        return

    try:
        red_state, red_count, red_last_change = red_alarm[0]
    except IndexError:
        return

    return JuniperAlarms(
        alarm_relay_mode=int(alarm_relay_mode),
        yellow_alarm=JuniperAlarm(
            state=int(yellow_state),
            count=int(yellow_count),
            last_change=int(yellow_last_change)
        ),
        red_alarm=JuniperAlarm(
            state=int(red_state),
            count=int(red_count),
            last_change=int(red_last_change)
        )
    )


def discovery_juniper_craft_alarms(section: JuniperAlarms) -> DiscoveryResult:
    yield Service()


def check_juniper_craft_alarms(params, section: JuniperAlarms) -> CheckResult:
    message = f'Alarm relay mode: {_juniper_alarm_relay_mode.get(section.alarm_relay_mode, section.alarm_relay_mode )}'
    if section.alarm_relay_mode != 2:  # pass on
        yield Result(state=State(params['state_not_pass_om']), summary=message)
    else:
        yield Result(state=State.OK, summary=message)

    message = f'Yellow alarm state: {_juniper_alarm_state.get(section.yellow_alarm.state, section.yellow_alarm.state)}'
    if section.yellow_alarm.state != 2:  # off
        yield Result(state=State(params['state_yellow_not_off']), summary=message)
    else:
        yield Result(state=State.OK, summary=message)

    message = f'Red alarm state: {_juniper_alarm_state.get(section.red_alarm.state, section.red_alarm.state)}'
    if section.red_alarm.state != 2:  # off
        yield Result(state=State(params['state_red_not_off']), summary=message)
    else:
        yield Result(state=State.OK, summary=message)

    yield from check_levels(
        value=section.yellow_alarm.count,
        label='Yellow alarm count',
        levels_upper=params.get('count_yellow'),
        render_func=lambda v: f'{v:.0f}',
        notice_only=True,
    )

    yield from check_levels(
        value=section.red_alarm.count,
        label='Red alarm count',
        levels_upper=params.get('count_red'),
        render_func=lambda v: f'{v:.0f}',
        notice_only=True,
    )

    yield Result(state=State.OK, notice=f'Yellow alarm last change: {section.yellow_alarm.last_change}')
    yield Result(state=State.OK, notice=f'Red alarm last change: {section.red_alarm.last_change}')


register.snmp_section(
    name='juniper_craft_alarms',
    parse_function=parse_juniper_craft_alarms,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2636.3.4.2',  # JUNIPER-ALARM-MIB::jnxCraftAlarms
            oids=[
                '1',  # jnxAlarmRelayMode
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2636.3.4.2.2',  # JUNIPER-ALARM-MIB::jnxCraftAlarms
            oids=[
                '1',  # jnxYellowAlarmState
                '2',  # jnxYellowAlarmCount
                '3',  # jnxYellowAlarmLastChange
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2636.3.4.2.3',  # JUNIPER-ALARM-MIB::jnxCraftAlarms
            oids=[
                '1',  # jnxRedAlarmState
                '2',  # jnxRedAlarmCount
                '3',  # jnxRedAlarmLastChange
            ]
        ),
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2636.1.1.1'),
)

register.check_plugin(
    name='juniper_craft_alarms',
    service_name='Chassis Alarm',
    discovery_function=discovery_juniper_craft_alarms,
    check_function=check_juniper_craft_alarms,
    check_ruleset_name='juniper_craft_alarms',
    check_default_parameters={
        'state_not_pass_om': 1,
        'state_yellow_not_off': 1,
        'state_red_not_off': 2,
    }
)
